#include <FastLED.h>
#include <LiquidCrystal_I2C.h>

int r=152;
int g=0;
int b=10;

#define LED_PIN  6           
#define NUM_LEDS 30
#define LCD_COL  15
const int inputWindow = 200;
CRGB leds[NUM_LEDS];
int ss=0;
int buttonPin = 1;
int buttonStatus = 0;
long lastDebounceTime = 0;
long debounceTime = 0;
long debounceDelay = 300;
int factor = 0;
int state = 0;
int counter = 0;
unsigned long startMillis;
unsigned long currentMillis;
int displayLcdTimer = 0;

LiquidCrystal_I2C lcd(0x27,16,2);
byte customChar[8] = {
	0b00000,
	0b01010,
	0b11111,
	0b11111,
	0b01110,
	0b00100,
	0b00000,
	0b00000
};

void setup() {
  Serial.begin(9600);

  FastLED.addLeds<WS2812, LED_PIN, GRB>(leds, NUM_LEDS);
  for (int i = NUM_LEDS/2; i >= 0; i--) {
    leds[i] = CRGB(r,g,b);
    leds[NUM_LEDS-i] = CRGB(r,g,b);
    FastLED.show();
  }

  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  
  OCR1A = 31249; // compare match register 16MHz/256/2Hz-1 => 2Hz
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS12);    // 256 prescaler 
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt

  sei();
  buttonStatus = 0;
  lastDebounceTime = millis();

  // https://riptutorial.com/arduino/example/9856/interrupt-on-button-press
  pinMode(buttonPin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(buttonPin), myISR, FALLING); // trigger when button pressed, but not when released.
  state = 0;
  factor = 0;
  startMillis = millis();

  lcd.init(); //initialize the lcd
  lcd.backlight(); //open the backlight
  lcd.createChar(0, customChar);
  lcd.setCursor(0,0);
  lcd.print("Mode: Down Up");
}

void myISR() {
  if (digitalRead(buttonPin) == LOW && (millis() - lastDebounceTime) > debounceDelay) {
    while (digitalRead(buttonPin) == LOW) {
      // wait for release
    }
  }
}

void displayLcdMode() {
  lcd.clear();
  if (factor == 0) {
    lcd.setCursor(0,0);
    lcd.print("Mode: Down Up");
  } else if (factor == 1) {
    lcd.setCursor(0,0);
    lcd.print("Mode: Up Down");
  } else if (factor == 2) {
    lcd.setCursor(0,0);
    lcd.print("Mode: Int Ext");
  } else if (factor == 3) {
    lcd.setCursor(0,0);
    lcd.print("Mode: Ext Int");
  }
}
void displayHearts0(int noHearts) {
  for (int i = 0; i < noHearts; i++) {
    lcd.setCursor(i,1);
    lcd.write((byte)0);
  }
}

void displayHearts1(int noHearts) {
  for (int i = noHearts; i >= noHearts - 4; i--) {
    lcd.setCursor(i,1);
    lcd.write((byte)0);
  }
}

ISR(TIMER1_COMPA_vect) {
  currentMillis = millis();
  if (currentMillis - startMillis >= 10000) {
    displayLcdTimer = 1;
    state++;
    factor++;
    if (state > 3) {
        state = 0;
        factor = 0;
    }
    startMillis = currentMillis;
  }
}

// 0
void downUp (int s) {
  int noHearts = 0;

  if ((s>=40)&&(s<=45)) {
    for (int i = 0; i <= 4; i++)
      leds[i] = CRGB(255, 0 , 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if ((s>=45)&&(s<=50)) {
    for (int i = 0; i <= 7; i++)
      leds[i] = CRGB(255, 127 , 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if ((s>=55)&&(s<=60)) {
    for (int i = 0; i <= 10; i++)
      leds[i] = CRGB(255, 255, 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if ((s>=65)&&(s<=70)) {
    for (int i = 0; i <= 13; i++)
      leds[i] = CRGB(0, 255, 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=70)&&(s<=72)) {
    for (int i = 0; i <= 17; i++)
      leds[i] = CRGB(0, 0, 255);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=73)&&(s<=74)){
    for (int i = 0; i <= 20; i++)
      leds[i] = CRGB(75, 0, 130);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=75)&&(s<=77)) {
    for (int i = 0; i <= 23; i++)
      leds[i] = (148, 0, 211);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=78)&&(s<=79)) {
    for (int i = 0; i <= 27; i++)
      leds[i] = CRGB (224, 0, 255);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=80)&&(s<=82)) {
    for (int i = 0; i <= 30; i++)
      leds[i] = CRGB (255, 0, 96); 
    displayHearts0(noHearts);
  }

  // clear leds
  leds[NUM_LEDS] = CRGB (0,0,0);
  for (int i = 0; i < NUM_LEDS; i++) 
    leds[i] = leds[i + 1];
  lcd.print(" ");
}

// 1
void upDown(int s) {
  int noHearts = 0; 
  if ((s>=40)&&(s<=45)) {
    for (int i = NUM_LEDS - 1; i >= 27 ; i--) 
      leds[i] = CRGB(255, 0 , 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if ((s>=45)&&(s<=50)) {
    for (int i = 26; i >= 24 ; i--) 
      leds[i] = CRGB(255, 127 , 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if ((s>=55)&&(s<=60)) {
    for (int i = 23; i >= 21 ; i--) 
      leds[i] = CRGB(255, 255, 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if ((s>=65)&&(s<=70)) {
    for (int i = 20; i >= 18 ; i--) 
      leds[i] = CRGB(0, 255, 0);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=70)&&(s<=72)) {
    for (int i = 17; i >= 16; i--) 
      leds[i] = CRGB(0, 0, 255);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=73)&&(s<=74)){
      for (int i = 15; i >= 14 ; i--) 
        leds[i] = CRGB(75, 0, 130);
      displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=75)&&(s<=77)) {
    for (int i = 13; i >= 10 ; i--) 
      leds[i] = (148, 0, 211);
    displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=78)&&(s<=79)) {
      for (int i = 9; i >= 5; i--) 
        leds[i] = CRGB (224, 0, 255);
      displayHearts0(noHearts);
  }

  noHearts++;
  if((s>=80)&&(s<=82)) {
    for (int i = 4; i >= 0; i--) 
      leds[i] = CRGB (255, 0, 96); 
    displayHearts0(noHearts);
  }

  // clear leds
  leds[0] = CRGB (0,0,0);
  for (int i = NUM_LEDS - 1; i > 0; i--) 
    leds[i] = leds[i - 1];

  // clear hearts row
  lcd.print(" ");
}

// 2
void intExt (int s) {
 if((s>=70)&&(s<=72) || (s>=50)&&(s<=55))  {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b);
    leds[NUM_LEDS/2]=CRGB (r, g, b);
    displayHearts0(1);
  }
  else if((s>=73)&&(s<=74) ||(s>=40)&&(s<=45)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g -10 , b - 5);
    leds[NUM_LEDS/2]=CRGB (r, g - 10, b - 5);
    displayHearts0(2);
  }
  else if((s>=75)&&(s<=76)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r - 10, g, b - 10);
    leds[NUM_LEDS/2]=CRGB (r - 10, g, b - 10);
    displayHearts0(3);
  }
  else if((s>=78)&&(s<=79)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r - 20, g -20, b - 20);
    leds[NUM_LEDS/2]=CRGB (r - 20, g - 20, b - 20);
    displayHearts0(4);
  }

  else if((s>=80)&&(s<=81)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 25);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 25);
    displayHearts0(5);
  }
  else if((s>=82)&&(s<=83)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 30);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 30);
    displayHearts0(6);
  }
  else if((s>=84)&&(s<=85)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 35);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 35);
    displayHearts0(7);
  }
  else if((s>=86)&&(s<=87)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 40);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 40);
    displayHearts0(8);
  } 
  else if((s>=88)&&(s<=89))  {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 45);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 45);
    displayHearts0(9);

  } 
  else if((s>=90)&&(s<=91)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 50);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 50);
    displayHearts0(10);
  }
  else if((s>=92)&&(s<=93)) {
    leds[(NUM_LEDS/2)-1]=CRGB (r, g, b - 55);
    leds[NUM_LEDS/2]=CRGB (r, g, b - 55);
    displayHearts0(11);
  }
  else {
    leds[(NUM_LEDS/2)-1] = CRGB (0,0,0);
    leds[NUM_LEDS/2] = CRGB ( 0,0,0);
  }

  for (int i = 0; i <= ((NUM_LEDS/2)-2); i++) {
    leds[i] = leds[i+1];
    leds[NUM_LEDS-1-i] = leds[NUM_LEDS-i-2];
  }
  lcd.print(" ");
}

// 3
void extInt (int s) {
 if((s>=70)&&(s<=72) || (s>=55)&&(s<=60))  {
        leds[0]=CRGB(255, 0 , 0);
        leds[NUM_LEDS-1]=CRGB(255, 0 , 0);
        displayHearts0(1);
      }
     else if((s>=73)&&(s<=74)){// ||(s>=40)&&(s<=45)) {
        leds[0]= CRGB(255, 127 , 0);
        leds[NUM_LEDS-1]= CRGB(255, 127 , 0);
        displayHearts0(2);
      }
      else if((s>=75)&&(s<=76)) {
        leds[0]= CRGB(255, 255, 0);
        leds[NUM_LEDS-1]= CRGB(255, 255, 0);
        displayHearts0(3);
      }
     else if((s>=78)&&(s<=79)) {
        leds[0]= CRGB(0, 255, 0);
        leds[NUM_LEDS-1]= CRGB(0, 255, 0);
        displayHearts0(4);
      }
      else if((s>=80)&&(s<=81)) {
        leds[0]= CRGB(0, 0, 255);
        leds[NUM_LEDS-1]= CRGB(0, 0, 255);
        displayHearts0(5);
      }
     else if((s>=82)&&(s<=83)) {
        leds[0]= CRGB(75, 0, 130);
        leds[NUM_LEDS-1]= CRGB(75, 0, 130);
        displayHearts0(6);
      }
      else if((s>=84)&&(s<=85)) {
        leds[NUM_LEDS-1]= CRGB(148, 0, 211);
        leds[0]= CRGB(148, 0, 211);
        displayHearts0(7);
      }
       else if((s>=86)&&(s<=87)) {
        leds[0]= CRGB(224, 0, 255);
        leds[NUM_LEDS-1]= CRGB(224, 0, 255);
        displayHearts0(8);
      }
      else if((s>=88)&&(s<=89))  {
        leds[NUM_LEDS-1]= CRGB (255, 0, 96); 
        leds[0]= CRGB (255, 0, 96); 
        displayHearts0(9);

      }else {
        leds[NUM_LEDS - 1] = CRGB (0,0,0);
        leds[0] = CRGB (0,0,0);
      }

      for (int i = NUM_LEDS / 2 - 2; i >= 0 ; i--) {
        leds[i + 1] = leds[i];
        leds[NUM_LEDS-i-2] = leds[NUM_LEDS-1-i];
      }
      lcd.print(" ");

}

void loop()
{
  unsigned int s = 0; // avgsound
  unsigned int inputMax = 0;
  unsigned int inputMin = 1024;
  unsigned int peakToPeak = 0;         // peak-to-peak level

  //------------------------------------------------------------------------------------ Get sound sample
  // loop for the window
  for (unsigned int i = 0; i < inputWindow; i++) {
    // read in a single value
    ss = analogRead(A0);

    // get the minimum and maximum value
    inputMin = min(inputMin, ss);
    inputMax = max(inputMax, ss);
    s += ss;
  }

  s = s / inputWindow;

 //------------------------------------------------------------------------------------ Read button + treat debounce
  if ((millis() - lastDebounceTime) > debounceDelay && digitalRead(buttonPin) == LOW) {
    if (buttonStatus == 1) {
      buttonStatus = 0;
      lastDebounceTime = millis();
    } else {
      buttonStatus = 1;
      lastDebounceTime = millis();
    }
    startMillis = currentMillis; // reset timer
    state++;
    factor++;
    if (state > 3) {
        state = 0;
        factor = 0;
    }
    // if button pressed change the text
    displayLcdMode();
  }

//------------------------------------------------------------------------------------- display LCD
  if (displayLcdTimer == 1) {
    displayLcdMode();
    displayLcdTimer = 0;
  }
//  Serial.println(displayLcdTimer);
//------------------------------------------------------------------------------------- display leds
  if (s <= 30) {
    for (int i = 0; i <NUM_LEDS; i++) 
      leds[i] = CRGB (0, 0, 0);
  } else {
//-------------------------------------------------------- interior spre exterior
// https://arduinogetstarted.com/tutorials/arduino-lcd-i2c
    if (factor == 2) {
     intExt(s);
//-------------------------------------------------------- din exterior spre interior
    } else if (factor == 3) { // out in
      extInt(s);

  //-------------------------------------------------------- de jos in sus
    } else if (factor == 0) {
      downUp(s);
// -------------------------------------------------- de sus in jos
    } else if (factor == 1) { 
      upDown(s);
    }
  }
 
 FastLED.show();
 
}
